package com.fatwire.cs.ui.view.renderer.attribute;

import java.util.List;

import javax.faces.component.UIComponent;

import oracle.adf.view.faces.component.core.layout.CorePanelGroup;
import oracle.adf.view.faces.component.core.output.CoreOutputText;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import COM.FutureTense.CS.Factory;
import COM.FutureTense.Interfaces.FTValList;
import COM.FutureTense.Interfaces.ICS;

import com.fatwire.assetapi.def.AttributeDef;
import com.fatwire.assetapi.def.AttributeTypeEnum;
import com.fatwire.cs.ui.exception.UIException;
import com.fatwire.cs.ui.model.bo.asset.AttributeEditor;
import com.fatwire.cs.ui.util.UILogDescriptions;
import com.fatwire.cs.ui.view.backing.GenericUtil;
import com.fatwire.cs.ui.view.jsf.ComponentFactory;
import com.fatwire.cs.ui.view.renderer.RenderConstants;
import com.fatwire.cs.ui.view.renderer.RenderDataContext;
import com.fatwire.cs.ui.view.util.EmbeddedLinkStreamWrapper;

/**
 * todo: Add class/interface details
 *
 * @author 
 */
public class StaticTextInspectRenderer extends BasicDataRenderer
{
    private Log log = LogFactory.getLog (UILogDescriptions.VIEW_LOG);

    public List<UIComponent> renderData(RenderDataContext renderContext) throws UIException
    {
        AttributeDef def = renderContext.getAttributeDef ();
        List data = renderContext.getAttributeData ().getDataAsList ();
        CorePanelGroup vPanel = (CorePanelGroup) ComponentFactory.create (CorePanelGroup.COMPONENT_TYPE);
        vPanel.setLayout (CorePanelGroup.LAYOUT_VERTICAL);
        boolean dataAvailable = false;
        for (int x = 0; data != null && x < data.size (); x++)
        {
            dataAvailable = true;
            Object fieldData = data.get (x);
            AttributeEditor editor = renderContext.getEditor ();
            if (!_isPassword (editor))
            {
                //not a password.. check if it is status field
                if (_isStatusField (def))
                {
                    fieldData = _getStatusData (fieldData);
                }

                // If datatype is Money, need to format currency to Default 
                // specific Locale
                if (_isMoneyType (def))
                {
                    fieldData = GenericUtil.getLocalizedMoneyFormat (fieldData);
                }

                // If field Data is not String, JSF will throw some error if the data 
                // is not serializable.So do a .String() on it.Anyways this ir CoreOutputText which is
                // going to display String.
                if (!(fieldData instanceof String))
                {
                    log.debug ("Using StaticTextInspectRenderer when the data type is not String. AttributeName : "
                                    + renderContext.getAttributeDef ().getName ());
                    fieldData = fieldData.toString ();
                }
                EmbeddedLinkStreamWrapper link = new EmbeddedLinkStreamWrapper (GenericUtil.getICS (), false, false);
                String fontStart = "<span class=\"" + renderContext.getCtrlStyle () + "\">";
                String fontEnd = "</span>";
                //normal case.  Just display as text the value   
                String outputData = (String) fieldData;
                CoreOutputText text;
                if (isEmpty (outputData))
                {
                   
                    text = getNoValueText ();
                }
                else
                {
                    text = createText (fontStart + link.Stream (outputData) + fontEnd);
                }
                text.setId (generateID (x));
                text.setEscape (false);
                vPanel.getChildren ().add (text);
            }
            else
            {
                //TODO not a very secure way of hiding password...admin only has to switch to a different attribute renderer
                vPanel.getChildren ().add (_createPasswordValue (fieldData));
            }
        }
        if(!dataAvailable)
        {
            vPanel.getChildren ().add (getNoValueText ());
        }

        return createList (vPanel);
    }





    // return true if this is the status attribute
    private boolean _isStatusField(AttributeDef def)
    {
        return "status".equals (def.getName ()) && def.isMetaDataAttribute ();
    }

    // return the status data
    private Object _getStatusData(Object fieldData) throws UIException
    {
        try
        {
            // Status attribute. Need to translate the value.
            ICS ics = Factory.newCS ();
            FTValList args = new FTValList ();
            args.setValString ("KEY", "dvin/AT/Common/StatusCode-" + fieldData.toString ());
            args.setValString ("VARNAME", "_statuscodedescription_");
            ics.runTag ("XLAT.LOOKUP", args);
            fieldData = ics.GetVar ("_statuscodedescription_");
        }
        catch (Exception e)
        {
            throw new UIException (e);
        }
        return fieldData;
    }

    // return true if this is a password type
    private boolean _isPassword(AttributeEditor editor)
    {
        return GenericUtil.isTrue (editor.getProperty (RenderConstants.TEXTFIELD.BLANKED));
    }

    //returns stars to display instead of the actual value;
    private CoreOutputText _createPasswordValue(Object fieldData)
    {
        if (fieldData == null)
        {
            return createText ("");
        }
        else
        {
            StringBuilder sb = new StringBuilder ();
            String value = fieldData.toString ();
            for (int i = 0; i < value.length (); i++)
            {
                sb.append ("*");
            }
            return createText (sb.toString ());
        }
    }

    //returns true if this is a money datatype field
    private boolean _isMoneyType(AttributeDef def)
    {
        if (AttributeTypeEnum.MONEY == def.getType ())
            return true;
        else
            return false;
    }

}
