/*
  * $Logfile:$ $Revision:$ $Date:$
  *
  * Copyright (c) 2006 FatWire Corporation, All Rights Reserved.
  */

/*
 * Copyright 2005 FatWire Corporation. Title, ownership rights, and
 * intellectual property rights in and to this software remain with
 * FatWire Corporation. This software is protected by international
 * copyright laws and treaties, and may be protected by other law.
 * Violation of copyright laws may result in civil liability and
 * criminal penalties.
 */
package com.fatwire.cs.ui.view.renderer.attribute;

import com.fatwire.assetapi.def.AttributeDef;
import com.fatwire.assetapi.def.AttributeTypeEnum;
import com.fatwire.cs.ui.exception.UIException;
import com.fatwire.cs.ui.exception.UIPropertyException;
import com.fatwire.cs.ui.model.bo.asset.*;
import com.fatwire.cs.ui.resource.ErrorConstants;
import com.fatwire.cs.ui.view.jsf.ComponentFactory;
import com.fatwire.cs.ui.view.renderer.AssetRenderUtil;
import com.fatwire.cs.ui.view.renderer.RenderConstants;
import com.fatwire.cs.ui.view.backing.GenericUtil;
import oracle.adf.view.faces.component.core.input.CoreInputText;

import javax.faces.component.UIComponent;
import java.util.List;

/*
Attribute editor properties supported:
======================================
XSIZE                               - Size of the control itself (number of colums in this case)
MAXCHARS                            - max number of characters.
BLANKED (true/false) default=false  - true indicates that control has to be rendered like a password
DEFAULT                             - Ignore it, data will have the default
INSTRUCTION                         - Tool tip or message below the control
*

/*
Type mapping:
=============
STRING          = Simply displays the data in editable form.
LONG            = Displays the value and associated a validator for Long
DATE            = Displays the value of text (formatted) and associated a validator for date
LARGE_TEXT      = Same as STRING
ASSET           = Shows id part of Asset Id (just the number)
FLOAT           = Displays the value and associated a validator for Float
MONEY           = Uses Content Server specific formatting (CURRENCY tags)
BLOB            = Old UI simply shows a blank text field, Now Invalid.
URL             = Old UI simply shows a blank text field, Now Invalid.
*/

/**
 * This renderer shows TextField's data.
 *
 * @author 
 */

/*
 * This implementation leaves inspect mode rendering to the base class.
 */
public class TextDataRenderer extends BasicDataTemplate
{

    private static final int DEFAULT_LENGTH_INTEGER_FIELD = 35;
    private static final int DEFAULT_LENGTH = 64;
    private static final int DEFAULT_COLS = 97;
    private static final int DEFAULT_INTEGER_COLS = 31;


    /**
     * Builds a InputText according to the properties specified in the UIAttributeEditor
     *
     * @return InputText with properties set
     */
    protected List<UIComponent> createComponents()
    {
        AttributeDef def = getContext().getAttributeDef();
        AttributeEditor editor = getContext().getEditor();

        // the control
        CoreInputText cit = (CoreInputText)ComponentFactory.create(CoreInputText.COMPONENT_TYPE);
        cit.setSimple(true);
        cit.setId(generateID());
        _addEditorProperties(editor, cit, def, getContext().getCtrlStyle() );


        AssetRenderUtil.addStandardConverters(cit, def);
        AssetRenderUtil.addValueBinding(cit, getContext().getBindingPattern(), getDataIndex());
        
        return createList(cit);
    }

    private void _addEditorProperties(AttributeEditor editor, CoreInputText cit, AttributeDef def, String ctrlStyle)
    {
        // XSIZE
        int xsz = GenericUtil.getValidInt((String)editor.getProperty(RenderConstants.TEXTFIELD.XSIZE));
        AttributeTypeEnum type = def.getType();
        boolean isNumberOrMoney = AssetUtil.isNumberType(type) || AttributeTypeEnum.MONEY == type;
        if(-1 != xsz)
        {
            cit.setColumns(xsz);
        }
        else
        {
            if(type != null && isNumberOrMoney )
            {
                cit.setColumns(DEFAULT_INTEGER_COLS);
               
            }
            else
            {
                cit.setColumns(DEFAULT_COLS);
            }
        }

        //MAXCHARS
        int maxchars = GenericUtil.getValidInt((String)editor.getProperty(RenderConstants.TEXTFIELD.MAXCHARS));
        int storageSize = def.getProperties().getSize();
		if(-1 != maxchars)
        {
			//If maxchars and storage size are available, make sure we set the maximum number of input characters
			//is the minimum of the both.
        	if(storageSize >= 0 )
        		if(maxchars <= storageSize)
        			cit.setMaximumLength(maxchars);
        		else
        			cit.setMaximumLength(storageSize);
        	else
        		cit.setMaximumLength(maxchars);	//if storageSize is not available, set the maxchars
        }
        else
        {
        	cit.setMaximumLength(storageSize <= 0 ? isNumberOrMoney ?DEFAULT_LENGTH_INTEGER_FIELD :DEFAULT_LENGTH : storageSize);
        }

        // INSTRUCTION
        String instruction = (String)editor.getProperty(RenderConstants.TEXTFIELD.INSTRUCTION);
        if(null != instruction && instruction.length() > 0)
        {
            cit.setTip(instruction);
        }

        // BLANKED
        if(GenericUtil.isTrue(editor.getProperty(RenderConstants.TEXTFIELD.BLANKED)))
        {
            cit.setSecret(true);
        }

        if(null != ctrlStyle)
        {
            cit.setStyleClass(ctrlStyle);
        }
    }


    protected void validateDataType(AttributeDef def) throws UIException
    {
        AttributeTypeEnum type = def.getType();
        if(AttributeTypeEnum.BLOB == type || AttributeTypeEnum.URL == type)
        {
            throw new UIPropertyException(ErrorConstants.BASIC_DATA_RENDERER__ILLEGALDATATYPE, def.getName(), type);
        }
    }



}
